/**<pre>
 * Title: GPS Simulator
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package gui.tray;

import gui.*;
import util.*;
import map.*;

import java.awt.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.table.*;
import javax.swing.event.*;




/* ====================================================================== */
/** This class is the People tab in the TrayDialog.  It displays a list
 *  of all users in the simulation, and enables adding and removing of users.
 *
 */
public class PnlTabFloors extends JPanel implements ListSelectionListener{

	/** The simulation controller that keeps all data. */
	private MapPanel map;

	/** Layout used in this panel. */
    private BorderLayout bLayMain = new BorderLayout();

    /** Layout used in this panel. */
    private GridBagLayout gbagLayInfo = new GridBagLayout();

    //Components in this JPanel

    /** Panel for buttons */
    private JPanel btnPanel = new JPanel();

    /** Add user button. */
    private JButton btnAdd = new JButton(" Add ");

    /** REmove user button. */
    private JButton btnRemove = new JButton("Remove");

    /** Scrollpane for JTable */
    private JScrollPane spnUsers = new JScrollPane();

    /** JTable for displaying users */
    private JTable tableFloors = new JTable();

    /** The JTable's columns. */
    private String[] colNames = { "Number" };

	/** A popup dialog for adding new user. */
	private JDialog addDialog;

    /** Field for user port number */
    private JTextField port_field;

    /** Field for user port number */
    private JTextField name_field;

    /** Field for uer range. */
    private JTextField range_field;

	private PnlTabs tabs;

    // ======================================================================
    /** Constructor sets up this Panel for a tab and calls init.
     *  It also signs us up as a controller listener.
     *
     */
    public PnlTabFloors( PnlTabs tabs, MapPanel map ) {

		this.map = map;
		this.tabs = tabs;
		//map.addControllerListener(this);

        try {
            init();
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }


    } //Constructor




    // ======================================================================
    /** Initializes this JPanel with Components.  Adds the table, buttons,
     *  and sets the action listeners.
     *
     */
    private void init() throws Exception {

		// Set the borders and layout
        bLayMain.setHgap(5);
        bLayMain.setVgap(5);
        this.setBorder(TTBorder.getBorder("Floors"));
        this.setLayout(bLayMain);
        btnPanel.setLayout(gbagLayInfo);

		//  Add stuff to the panels and table
        this.add(btnPanel, BorderLayout.SOUTH);

        btnPanel.add(btnAdd, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(5, 10, 0, 0), 0, 0));

        btnPanel.add(btnRemove, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(5, 10, 0, 0), 0, 0));


		// Set up the table and scrollpane
        this.add(spnUsers,  BorderLayout.CENTER);

		String[][] rowData = new String[0][colNames.length]; // The data array


        DefaultTableModel dtm = new DefaultTableModel(rowData, colNames) {
            // set this to non-editable table model.
            public boolean isCellEditable(int row, int col) {
                return false;
            }
        };
        tableFloors = new JTable(dtm);

        spnUsers.getViewport().add(tableFloors, 0);


		// Setup the action listeners
		btnAdd.setMnemonic(KeyEvent.VK_A);
        btnAdd.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  btdAdd_actionPerformed();
              }
        });
		btnRemove.setMnemonic(KeyEvent.VK_R);
        btnRemove.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                btdRemove_actionPerformed();
            }
        });

		// Set panel size (gets over ridden by TrayDialog's set size)
        this.setSize(new Dimension(550, 300));

        refresh();

    } //method: init




    // ========================================================================
    /** This method is called to add a new user to the simulation.
     *
     */
    public void btdAdd_actionPerformed() {


		//Setup a current/new JDialog.
		addDialog = new JDialog(map.getMapFrame());
		addDialog.setTitle("Add Floor");

		// Create buttons and the labels.
		JLabel lbl = new JLabel("Enter the floor number below.");

		JButton btnAdd = new JButton( " ADD ");
		JButton btnCancel = new JButton( "CANCEL");

		JLabel namelbl = new JLabel(" Number: "  );
		name_field = new JTextField(15);
		name_field.setText(""+(map.getBuilding().getFloors().size()+1));

		//JLabel portlbl = new JLabel(" Port Number: "  );
		//port_field = new JTextField(15);

		//JLabel rangelbl = new JLabel(" Wireless Range: "  );
		//range_field = new JTextField(15);
		//range_field.setText("60");

		//Create panels for hold the Components
		JPanel portpanel = new JPanel(new GridBagLayout() );

		JPanel btnpanel = new JPanel(new GridBagLayout());
		JPanel lblpanel = new JPanel(new BorderLayout());
		lblpanel.setBorder( BorderFactory.createEmptyBorder(15, 15, 15, 15) );


        portpanel.add(namelbl, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(10, 0, 5, 0), 0, 0));

        portpanel.add(name_field, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(10, 5, 5, 0), 0, 0));



        btnpanel.add(btnAdd, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(2, 2, 2, 2), 0, 0));

        btnpanel.add(btnCancel, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 0, 0), 0, 0));


		//Add components to the panels, then add panels to the JDialog.


		lblpanel.add(lbl, BorderLayout.NORTH);
		lblpanel.add(portpanel, BorderLayout.EAST);

		addDialog.getContentPane().add(lblpanel, BorderLayout.CENTER);
		addDialog.getContentPane().add(btnpanel, BorderLayout.SOUTH);


        //range_field.addKeyListener(new java.awt.event.KeyAdapter() {
        //    public void keyPressed(KeyEvent e) {
        //        enterKeyPressed(e);
        //    }
        //});

        name_field.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                enterKeyPressed(e);
            }
        });

		//Make a button listener to close the dialog.
		btnAdd.setMnemonic(KeyEvent.VK_A);
		btnAdd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				btdDialogAdd_actionPerformed();
			}
		});

		btnCancel.setMnemonic(KeyEvent.VK_C);
		btnCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {

				addDialog.dispose();
			}
		});

		//Pack it all together
		addDialog.pack();

		//Center this dialog on the screen.
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		int x = ( screen.width / 2) - (addDialog.getSize().width /2 );
		int y = ( screen.height / 2) - (addDialog.getSize().height /2 );
		addDialog.setLocation( x, y);

		//Now show this dialog.
		addDialog.setVisible(true);

    } //method:




    // ========================================================================
    /** This method is called to remove a user from the simulation.
     *
     */
    public void btdRemove_actionPerformed() {

		TableModel tm = tableFloors.getModel();

		int row = tableFloors.getSelectedRow();

		String name = (String)tm.getValueAt(row, 0);

		Vector floors = map.getBuilding().getFloors();
		Floor use;

		for (int i=0; i < floors.size(); i++) {
			use = (Floor)floors.get(i);
			if(use.getNumber() == Integer.parseInt(name) ){
				floors.removeElement(use);
			}
		}

		refresh();

    } //method: btdRemove_actionPerformed



    // ========================================================================
    /** This method is
     *
     */
    public void btdDialogAdd_actionPerformed() {

		System.out.println("making a new floor...");

		//String p = port_field.getText();
		String n = name_field.getText();


		if( n.length() < 1 ){
			JOptionPane.showMessageDialog(addDialog,
				"Please enter a number.",
				"Illegal Floor Number",
				JOptionPane.INFORMATION_MESSAGE );
			addDialog.requestFocus();
			return;
		}

		try{
			int num = Integer.parseInt(n);
			//int port = Integer.parseInt(p);


			map.getBuilding().addFloor( new Floor(num) );


		}catch(Exception ex){
			JOptionPane.showMessageDialog(addDialog,
				"Please enter only numbers in the fields.",
				"Bad Input",
				JOptionPane.INFORMATION_MESSAGE );
			addDialog.requestFocus();
			return;
		}

		addDialog.dispose();

		refresh();

	}

     /* ======================================================================== */
    /** Called when the enter key is pressed in the text fields. This method is
     *  called by the KeyListener of the textfields.
     */
    private void enterKeyPressed(KeyEvent e) {
        int key = e.getKeyCode();
        if (key == KeyEvent.VK_ENTER) {
            btdDialogAdd_actionPerformed();
        }

    }


    // ========================================================================
    /** This method required by ControllerListener interface.
     *  This method resets the data in the JTable, so as to keep it up to date.
     *
     */
	public void refresh(){

		Vector floors = map.getBuilding().getFloors();
		Position pos;
		Floor use;

		String[][] rowData = new String[floors.size()][colNames.length]; // The data array

		for (int i=0; i < floors.size(); i++) {
			use = (Floor)floors.get(i);

	  		rowData[i][0] = ""+use.getNumber();

		}

        DefaultTableModel dtm = new DefaultTableModel(rowData, colNames) {
            // set this to non-editable table model.
            public boolean isCellEditable(int row, int col) {
                return false;
            }
        };
        tableFloors = new JTable(dtm);
        tableFloors.getSelectionModel().addListSelectionListener(this);

        spnUsers.getViewport().add(tableFloors, 0);


	} //method: refresh




	public void valueChanged(ListSelectionEvent e){
		int row = e.getFirstIndex();
		if( row > -1 ){
			try{
				tabs.getTabPane().removeTabAt(1);
				tabs.getTabPane().removeTabAt(1);
				tabs.getTabPane().removeTabAt(1);
				tabs.getTabPane().removeTabAt(1);
				tabs.getTabPane().removeTabAt(1);
				tabs.getTabPane().removeTabAt(1);
			}catch(Exception ee){}
			tabs.getTabPane().add(new PnlTabRooms( tabs, map, this ),  "Rooms");
		}else{
			try{
				tabs.getTabPane().removeTabAt(1);
				tabs.getTabPane().removeTabAt(1);
				tabs.getTabPane().removeTabAt(1);
				tabs.getTabPane().removeTabAt(1);
				tabs.getTabPane().removeTabAt(1);
				tabs.getTabPane().removeTabAt(1);
			}catch(Exception ee){}
		}

		map.setFloor(getFloor());
		map.setCurrent(getFloor());
		map.repaint();

	}


	public Floor getFloor(){

		int row = tableFloors.getSelectedRow();
		if( row < 0 ){ return null;
		}else{
			Vector floors = map.getBuilding().getFloors();
			return (Floor)(floors.elementAt(row));

		}
	}



} //class: PnlTabFloors