/**<pre>
 * Title: GPS Simulator
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package gui.tray;

import gui.*;
import util.*;
import map.*;

import java.awt.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.table.*;
import javax.swing.event.*;




/* ====================================================================== */
/** This class is the People tab in the TrayDialog.  It displays a list
 *  of all users in the simulation, and enables adding and removing of users.
 *
 */
public class PnlTabRooms extends JPanel implements ListSelectionListener{

	/** The simulation controller that keeps all data. */
	private MapPanel map;

	/** Layout used in this panel. */
    private BorderLayout bLayMain = new BorderLayout();

    /** Layout used in this panel. */
    private GridBagLayout gbagLayInfo = new GridBagLayout();

    //Components in this JPanel

    /** Panel for buttons */
    private JPanel btnPanel = new JPanel();

    /** Add user button. */
    private JButton btnAdd = new JButton(" Add ");

    /** REmove user button. */
    private JButton btnRemove = new JButton("Remove");

    /** Scrollpane for JTable */
    private JScrollPane spnUsers = new JScrollPane();

    /** JTable for displaying users */
    private JTable tableRooms = new JTable();

    /** The JTable's columns. */
    private String[] colNames = { "Temp", "Visibility", "O2 level", "dB level" };

	/** A popup dialog for adding new user. */
	private JDialog addDialog;

    /** Field for user port number */
    private JTextField oxy_field;

    /** Field for user port number */
    private JTextField name_field;

    /** Field for uer range. */
    private JTextField vis_field;

    private JTextField temp_field;

    private JTextField sound_field;

	private PnlTabs tabs;

	private PnlTabFloors parent;

	private Floor floor;

    // ======================================================================
    /** Constructor sets up this Panel for a tab and calls init.
     *  It also signs us up as a controller listener.
     *
     */
    public PnlTabRooms( PnlTabs tabs, MapPanel map, PnlTabFloors parent ) {

		this.map = map;
		this.tabs = tabs;
		this.parent = parent;
		//map.addControllerListener(this);

        try {
            init();
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }

    } //Constructor




    // ======================================================================
    /** Initializes this JPanel with Components.  Adds the table, buttons,
     *  and sets the action listeners.
     *
     */
    private void init() throws Exception {

		// Set the borders and layout
        bLayMain.setHgap(5);
        bLayMain.setVgap(5);
        this.setBorder(TTBorder.getBorder("Rooms"));
        this.setLayout(bLayMain);
        btnPanel.setLayout(gbagLayInfo);

		//  Add stuff to the panels and table
        this.add(btnPanel, BorderLayout.SOUTH);

        btnPanel.add(btnAdd, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(5, 10, 0, 0), 0, 0));

        btnPanel.add(btnRemove, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(5, 10, 0, 0), 0, 0));


		// Set up the table and scrollpane
        this.add(spnUsers,  BorderLayout.CENTER);

		String[][] rowData = new String[0][colNames.length]; // The data array


        DefaultTableModel dtm = new DefaultTableModel(rowData, colNames) {
            // set this to non-editable table model.
            public boolean isCellEditable(int row, int col) {
                return false;
            }
        };
        tableRooms = new JTable(dtm);

        spnUsers.getViewport().add(tableRooms, 0);


		// Setup the action listeners
		btnAdd.setMnemonic(KeyEvent.VK_A);
        btnAdd.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  btdAdd_actionPerformed();
              }
        });
		btnRemove.setMnemonic(KeyEvent.VK_R);
        btnRemove.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                btdRemove_actionPerformed();
            }
        });

		// Set panel size (gets over ridden by TrayDialog's set size)
        this.setSize(new Dimension(550, 300));

        refresh();

    } //method: init




    // ========================================================================
    /** This method is called to add a new user to the simulation.
     *
     */
    public void btdAdd_actionPerformed() {


		//Setup a current/new JDialog.
		addDialog = new JDialog(map.getMapFrame());
		addDialog.setTitle("Add Room");

		// Create buttons and the labels.
		JLabel lbl = new JLabel("Enter the room information below.");

		JButton btnAdd = new JButton( " ADD ");
		JButton btnCancel = new JButton( "CANCEL");

		JLabel namelbl = new JLabel(" Room Number: "  );
		name_field = new JTextField(15);
		name_field.setText("room"+(floor.getRooms().size()+1));

		JLabel vislbl = new JLabel(" Visibility: "  );
		vis_field = new JTextField(15);
		vis_field.setText("0.9");

		JLabel templbl = new JLabel(" Temperature: "  );
		temp_field = new JTextField(15);
		temp_field.setText("75");

		JLabel oxylbl = new JLabel(" Oxygen Level: "  );
		oxy_field = new JTextField(15);
		oxy_field.setText("0.9");

		JLabel soundlbl = new JLabel(" Sound Level: "  );
		sound_field = new JTextField(15);
		sound_field.setText("45");


		//Create panels for hold the Components
		JPanel portpanel = new JPanel(new GridBagLayout() );

		JPanel btnpanel = new JPanel(new GridBagLayout());
		JPanel lblpanel = new JPanel(new BorderLayout());
		lblpanel.setBorder( BorderFactory.createEmptyBorder(15, 15, 15, 15) );


        //portpanel.add(namelbl, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
        //            GridBagConstraints.WEST,
        //            GridBagConstraints.HORIZONTAL,
        //            new Insets(10, 0, 5, 0), 0, 0));

        //portpanel.add(name_field, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
        //            GridBagConstraints.WEST,
        //            GridBagConstraints.NONE,
        //            new Insets(10, 5, 5, 0), 0, 0));


        portpanel.add(templbl, new GridBagConstraints(0, 0, 1, 1, 0.5, 0.0,
                     GridBagConstraints.WEST,
                     GridBagConstraints.HORIZONTAL,
                    new Insets(10, 0, 5, 0), 0, 0));

        portpanel.add(temp_field, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(10, 5, 5, 0), 0, 0));

        portpanel.add(vislbl, new GridBagConstraints(0, 1, 1, 1, 0.5, 0.0,
                     GridBagConstraints.WEST,
                     GridBagConstraints.HORIZONTAL,
                    new Insets(10, 0, 5, 0), 0, 0));

        portpanel.add(vis_field, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(10, 5, 5, 0), 0, 0));

        portpanel.add(oxylbl, new GridBagConstraints(0, 2, 1, 1, 0.5, 0.0,
                     GridBagConstraints.WEST,
                     GridBagConstraints.HORIZONTAL,
                    new Insets(10, 0, 5, 0), 0, 0));

        portpanel.add(oxy_field, new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(10, 5, 5, 0), 0, 0));

        portpanel.add(soundlbl, new GridBagConstraints(0, 3, 1, 1, 0.5, 0.0,
                     GridBagConstraints.WEST,
                     GridBagConstraints.HORIZONTAL,
                    new Insets(10, 0, 5, 0), 0, 0));

        portpanel.add(sound_field, new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(10, 5, 5, 0), 0, 0));


        btnpanel.add(btnAdd, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(2, 2, 2, 2), 0, 0));

        btnpanel.add(btnCancel, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 0, 0), 0, 0));


		//Add components to the panels, then add panels to the JDialog.


		lblpanel.add(lbl, BorderLayout.NORTH);
		lblpanel.add(portpanel, BorderLayout.EAST);

		addDialog.getContentPane().add(lblpanel, BorderLayout.CENTER);
		addDialog.getContentPane().add(btnpanel, BorderLayout.SOUTH);


        oxy_field.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                enterKeyPressed(e);
            }
        });
        vis_field.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                enterKeyPressed(e);
            }
        });
        temp_field.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                enterKeyPressed(e);
            }
        });
        //name_field.addKeyListener(new java.awt.event.KeyAdapter() {
        //    public void keyPressed(KeyEvent e) {
        //        enterKeyPressed(e);
        //    }
        //});
        sound_field.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                enterKeyPressed(e);
            }
        });

		//Make a button listener to close the dialog.
		btnAdd.setMnemonic(KeyEvent.VK_A);
		btnAdd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				btdDialogAdd_actionPerformed();
			}
		});

		btnCancel.setMnemonic(KeyEvent.VK_C);
		btnCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {

				addDialog.dispose();
			}
		});

		//Pack it all together
		addDialog.pack();

		//Center this dialog on the screen.
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		int x = ( screen.width / 2) - (addDialog.getSize().width /2 );
		int y = ( screen.height / 2) - (addDialog.getSize().height /2 );
		addDialog.setLocation( x, y);

		//Now show this dialog.
		addDialog.setVisible(true);

    } //method:




    // ========================================================================
    /** This method is called to remove a user from the simulation.
     *
     */
    public void btdRemove_actionPerformed() {

		floor = parent.getFloor();

		TableModel tm = tableRooms.getModel();

		int row = tableRooms.getSelectedRow();
		Vector rooms = floor.getRooms();

		rooms.removeElementAt(row);

		///String name = (String)tm.getValueAt(row, 0);


		//Room use;

		//for (int i=0; i < rooms.size(); i++) {
		//	use = (Room)rooms.get(i);
		//	if(use.getName().equals(name) ){
		//		rooms.removeElement(use);
		//	}
		//}

		refresh();
		map.repaint();

    } //method: btdRemove_actionPerformed



    // ========================================================================
    /** This method is
     *
     */
    public void btdDialogAdd_actionPerformed() {

		floor = parent.getFloor();

		System.out.println("making a new room...");

		String o = oxy_field.getText();
		String v = vis_field.getText();
		String t = temp_field.getText();
		String n = sound_field.getText();


		if( n.length() < 1 ){
			JOptionPane.showMessageDialog(addDialog,
				"Please enter a sound level.",
				"Illegal Room sound level",
				JOptionPane.INFORMATION_MESSAGE );
			addDialog.requestFocus();
			return;
		}
		if( o.length() < 1 ){
			JOptionPane.showMessageDialog(addDialog,
				"Please enter a number.",
				"Illegal O2 Number",
				JOptionPane.INFORMATION_MESSAGE );
			addDialog.requestFocus();
			return;
		}
		if( v.length() < 1 ){
			JOptionPane.showMessageDialog(addDialog,
				"Please enter a number.",
				"Illegal visiiblity Number",
				JOptionPane.INFORMATION_MESSAGE );
			addDialog.requestFocus();
			return;
		}
		if( t.length() < 1 ){
			JOptionPane.showMessageDialog(addDialog,
				"Please enter a number.",
				"Illegal temp Number",
				JOptionPane.INFORMATION_MESSAGE );
			addDialog.requestFocus();
			return;
		}

		try{
			double oxy = Double.parseDouble(o);
			double vis = Double.parseDouble(v);
			double temp = Double.parseDouble(t);
			double sound = Double.parseDouble(n);


			floor.addRoom( new Room(temp, oxy, vis, sound) );


		}catch(Exception ex){
			JOptionPane.showMessageDialog(addDialog,
				"Please enter only numbers in the fields.",
				"Bad Input",
				JOptionPane.INFORMATION_MESSAGE );
			addDialog.requestFocus();
			return;
		}

		addDialog.dispose();

		refresh();

	}

     /* ======================================================================== */
    /** Called when the enter key is pressed in the text fields. This method is
     *  called by the KeyListener of the textfields.
     */
    private void enterKeyPressed(KeyEvent e) {
        int key = e.getKeyCode();
        if (key == KeyEvent.VK_ENTER) {
            btdDialogAdd_actionPerformed();
        }

    }


    // ========================================================================
    /** This method required by ControllerListener interface.
     *  This method resets the data in the JTable, so as to keep it up to date.
     *
     */
	public void refresh(){

		floor = parent.getFloor();

		Vector rooms = floor.getRooms();
		Position pos;
		Room use;

		String[][] rowData = new String[rooms.size()][colNames.length]; // The data array

		for (int i=0; i < rooms.size(); i++) {
			use = (Room)rooms.get(i);


	  		rowData[i][0] = ""+use.getTemperature();
	  		rowData[i][1] = ""+use.getVisibility();
	  		rowData[i][2] = ""+use.getOxygen();
	  		rowData[i][3] = ""+use.getSound();

		}

        DefaultTableModel dtm = new DefaultTableModel(rowData, colNames) {
            // set this to non-editable table model.
            public boolean isCellEditable(int row, int col) {
                return false;
            }
        };
        tableRooms = new JTable(dtm);
        tableRooms.getSelectionModel().addListSelectionListener(this);

        spnUsers.getViewport().add(tableRooms, 0);


	} //method: refresh



 	public void valueChanged(ListSelectionEvent e){

		//System.out.println("asdfas fasdf asdf dfas f");

		int row = e.getFirstIndex();
		if( row > -1 ){
			try{
				tabs.getTabPane().removeTabAt(2);
				tabs.getTabPane().removeTabAt(2);
				tabs.getTabPane().removeTabAt(2);
			}catch(Exception ee){}
			tabs.getTabPane().add(new PnlTabWalls( tabs, map, this ),  "Walls");
			tabs.getTabPane().add(new PnlTabFurnishings( tabs, map, this ),  "Furnishings");
		}else{
			try{
				tabs.getTabPane().removeTabAt(2);
				tabs.getTabPane().removeTabAt(2);
				tabs.getTabPane().removeTabAt(2);
			}catch(Exception ee){}
		}

		map.setCurrent(getRoom());
		map.repaint();

	}



	public Room getRoom(){

		floor = parent.getFloor();

		int row = tableRooms.getSelectedRow();

		if( row < 0 ){ return null;
		}else{
			Vector rooms = floor.getRooms();
			return (Room)(rooms.elementAt(row));
		}

	}



} //class: PnlTabFloors